*:*********************************************************************
*:
*: Procedure file: C:\DATA\FBEST\AEDIT.PRG
*:
*:         System: AEDIT
*:         Author: Stphane Desnault
*:      Copyright (c) 1993, SOFTWARE ASSETS
*:  Last modified: 02/25/93     10:45
*:
*:  Procs & Fncts: REDRAW()
*:               : DRAWCURSOR()
*:               : MOUSEPRESS()
*:               : MOVERIGHT()
*:               : MOVELEFT()
*:               : MOVEDOWN()
*:               : MOVEUP()
*:               : SCREENHOME()
*:               : SCREENEND()
*:               : SCREENRIGHT()
*:               : SCREENLEFT()
*:               : SCREENUP()
*:               : SCREENDOWN()
*:               : EDITCELL()
*:               : QUESTION()
*:               : CLEARCURSOR()
*:               : DRAWELEV()
*:
*:          Calls: REDRAW()           (function  in AEDIT.PRG)
*:               : DRAWCURSOR()       (function  in AEDIT.PRG)
*:               : MOUSEPRESS()       (function  in AEDIT.PRG)
*:               : MOVERIGHT()        (function  in AEDIT.PRG)
*:               : MOVELEFT()         (function  in AEDIT.PRG)
*:               : MOVEDOWN()         (function  in AEDIT.PRG)
*:               : MOVEUP()           (function  in AEDIT.PRG)
*:               : SCREENHOME()       (function  in AEDIT.PRG)
*:               : SCREENEND()        (function  in AEDIT.PRG)
*:               : SCREENRIGHT()      (function  in AEDIT.PRG)
*:               : SCREENLEFT()       (function  in AEDIT.PRG)
*:               : SCREENUP()         (function  in AEDIT.PRG)
*:               : SCREENDOWN()       (function  in AEDIT.PRG)
*:               : EDITCELL()         (function  in AEDIT.PRG)
*:               : QUESTION()         (function  in AEDIT.PRG)
*:
*:      Documented 02/25/93 at 11:16               FoxDoc  version 2.10b
*:*********************************************************************
PARAMETERS laArr,lcPicture,laLeftT,laTopT,lnVCnt, lnHCnt

* laArr     : Array being edited.
* lcPicture : Picture clause to display and edit an individual cell.
* laLeftT   : -OPTIONAL- Array of line titles to display on the 
*             left of the array. Defaults to line numbers.
* laTopT    : -OPTIONAL- Array of column titles to display above the
*             array. Defaults to sequential letters, as in a 
*             spreadsheet (e.g: A,B..,Y,Z,AA,AB...).
* lnVCnt    : -OPTIONAL- Number of lines of the array to display.
*             Defaults to the maximum number of lines that will fit
*             on the screen.
* lnHCnt    : -OPTIONAL- Number of columns of the array to display.
*             Defaults to the maximum number of columns that will
*             fit on the screen.

PRIVATE lnTopRow    && Array index of the top-most displayed row.

PRIVATE lnLeftCol   && Array index of the left-most displayed row.

PRIVATE lnRow       && Current row of the cursor, relative to the 
                    && displayed rows (e.g. if the cursor is on the 
                    && top of the display, lnRow=1).

PRIVATE lnCol       && Current column of the cursor, relative to the 
                    && displayed columns (e.g. if the cursor is on 
                    && the left of the display, lnCol=1).
PRIVATE i,j         && i loops on rows, j loops on columns.

PRIVATE lnCellLen   && Length of a displayed array element.

PRIVATE lnLeftT     && Length of the titles on the left.

PRIVATE lnRowLen    && Total length of a displayed row

PRIVATE llExit      && Main event loop flag

PRIVATE lnHElev     && Position of the horizontal elevator.
                    && 0 indicates the leftmost position,
                    && WCOLS("editArray")-1 the rightmost one.
                    
PRIVATE lnVElev     && Position of the vertical elevator.
                    && 0 indicates the leftmost position,
                    && WCOLS("editArray")-1 the rightmost one.
                    
PRIVATE lcCompStat  && Entry status of SET COMPATIBLE
PRIVATE lcTalkStat  && Entry status of SET TALK
PRIVATE lcEscStat   && Entry status of SET ESCAPE
PRIVATE lcCurStat   && Entry status of SET CURSOR

PRIVATE lnKey       && Key code for the main event loop.

PRIVATE llUpdate    && True when the array has been modified

PRIVATE lcResult    && True when the user confirms his modifications
                    && to the array before exiting the program.

* Save the environment

lcCompStat = SET("COMPATIBLE")
SET COMPATIBLE FOXPLUS

lcTalkStat = SET("TALK")
SET TALK OFF

lcEscStat = SET("ESCAPE")
SET ESCAPE OFF

lcCurStat = SET("CURSOR")
SET CURSOR OFF

* Save a backup copy of the array we are editing

DIMENSION laBackup(ALEN(laArr,1),ALEN(laArr,2))
=ACOPY(laArr,laBackup)

* Initialize the title arrays if the user didn't pass them
* as parameters.

* Default line titles (1,2,3...)
IF TYPE("laLeftT")<>"A"
  DIMENSION laLeftT[ALEN(laArr,1),1]
  FOR i=1 TO ALEN(laLeftT)
    laLeftT[i]=STR(i,3)
  ENDFOR
ENDIF

* Default column titles (A,B,..,Y,Z,AA,AB...)
IF TYPE("laTopT")<>"A"
  DIMENSION laTopT[ALEN(laArr,2),1]
  FOR j=1 TO ALEN(laTopT)
    laTopT[j]=IIF(j<=26,CHR(j+64),CHR(((j-1)/26)+64)+CHR(((j-1)%26)+65))
  ENDFOR
ENDIF

* Compute display parameters

* Length of a displayed array element
lnCellLen=LEN(TRANSFORM(laArr[1,1],lcPicture))+1

* Width of the left title column
lnLeftT=0
FOR i=1 TO ALEN(laLeftT,1)
  lnLeftT=MAX(lnLeftT,LEN(laLeftT[i]))
ENDFOR

* Number of columns of the array to display
IF (TYPE("lnHCnt")<>"N") .OR. (lnHCnt>ALEN(laArr,2))
  lnHCnt=ALEN(laArr,2)
ENDIF

lnRowLen=lnLeftT+lnHCnt*lnCellLen
IF lnRowLen>(SCOLS()-4)
  lnHCnt=FLOOR((SCOLS()-4-lnLeftT)/lnCellLen)
  lnRowLen=lnLeftT+lnHCnt*lnCellLen
ENDIF

* Number of lines of the array to display
IF (TYPE("lnVCnt")<>"N") .OR. (lnVCnt>ALEN(laArr,1))
  lnVCnt=ALEN(laArr,1)
ENDIF

IF lnVCnt>(SROWS()-4)
  lnVCnt=(SROWS()-4)
ENDIF

* Define main editing window
DEFINE WINDOW editArray ;
  FROM INT((SROW()-(lnVCnt+4))/2),INT((SCOL()-(lnRowLen+4))/2) ;
  TO INT((SROW()-(lnVCnt+4))/2)+(lnVCnt+3), ;
  INT((SCOL()-(lnRowLen+4))/2)+(lnRowLen+3) ;
  FLOAT ;
  NOCLOSE ;
  NONE ;
  COLOR SCHEME 8

ACTIVATE WINDOW editarray

* Initialize display variables
lnTopRow=1
lnRow=1
lnLeftCol=1
lnCol=1
llExit=.F.
lnHElev=0
lnVElev=0
lcResult="Unchanged"

* Draw window borders
@         0,        0 FILL TO         0,WCOLS()-1 COLOR (SCHEME(9,10))
@         0,WCOLS()-1 FILL TO WROWS()-1,WCOLS()-1 COLOR (SCHEME(9,10))
@ WROWS()-1,        0 FILL TO WROWS()-1,WCOLS()-1 COLOR (SCHEME(9,10))
@         0,        0 FILL TO WROWS()-1,        0 COLOR (SCHEME(9,10))

* If horizontal scrolling is needed, create and
* display horizontal elevator
IF lnHCnt < ALEN(laArr,2)
  @ WROWS()-1,        1 SAY "" COLOR (SCHEME(9,10))
  @ WROWS()-1,WCOLS()-2 SAY "" COLOR (SCHEME(9,10))
  DEFINE WINDOW wHElev ;
    IN editArray ;
    FROM WROWS()-1,2 TO WROWS()-1,2 ;
    NONE ;
    COLOR SCHEME 9
  ACTIVATE WINDOW wHElev
  @ 0,0 SAY "" COLOR (SCHEME(9,10))
  ACTIVATE WINDOW editarray
ENDIF

* If vertical scrolling is needed, create and
* display vertical elevator
IF lnVCnt < ALEN(laArr,1)
  @         1,WCOLS()-1 SAY "" COLOR (SCHEME(9,10))
  @ WROWS()-2,WCOLS()-1 SAY "" COLOR (SCHEME(9,10))
  DEFINE WINDOW wVElev ;
    IN editArray ;
    FROM 2,WCOLS()-1 TO 2,WCOLS()-1 ;
    NONE ;
    COLOR SCHEME 9
  ACTIVATE WINDOW wVElev
  @ 0,0 SAY "" COLOR (SCHEME(9,10))
  ACTIVATE WINDOW editarray
ENDIF

* Display "Close" icon on the top left corner
@ 0,0 SAY "" COLOR (SCHEME(9,10))

* Draw array limits in the window
@ 2,1 TO 2,WCOLS()-2
@ 2,lnLeftT+2 SAY ""
IF lnVCnt=1
	@ 3,lnLeftT+2 SAY ""
ELSE
	@ 3,lnLeftT+2 TO WROWS()-2,lnLeftT+2
ENDIF

* Display data
=redraw("HV")
=drawCursor()

* Main edition loop

DO WHILE ! llExit
  
  lnKey=INKEY("M")                         
                                           
  DO CASE                                  
    CASE lnKey=0                           && No key pressed
      LOOP                                 
    CASE lnKey=151                         && Left mouse button
      =mousePress()                        
    CASE lnKey=4                           && Right
      =moveRight()                         
    CASE lnKey=19 OR lnKey=127             && Left
      =moveLeft()                          
    CASE lnKey=24                          && Down
      =moveDown()                          
    CASE lnKey=5                           && Up
      =moveUp()                            
    CASE lnKey=1                           && Home
      =screenHome()                        
    CASE lnKey=6                           && End
      =screenEnd()                         
    CASE lnKey=9                           && Tab
      =screenRight()                       
    CASE lnKey=15                          && BackTab
      =screenLeft()                        
    CASE lnKey=18                          && Page up
      =screenUp()                          
    CASE lnKey=3                           && Page down
      =screenDown()                        
    CASE lnKey=13                          && Enter
      =editCell()                          
    CASE lnKey=10                          && Ctrl-Enter
      =exitAedit()                           
    CASE lnKey=27                          && Escape
      =exitAedit()
    CASE BETWEEN(lnKey,48,57) OR lnKey=45  && "-" or digit
      KEYBOARD CHR(lnKey)                  
      =editCell()                          
  ENDCASE
ENDDO

* Clean up after ourselves...

RELEASE WINDOW editArray
RELEASE WINDOW wVElev
RELEASE WINDOW wHElev

SET TALK &lcTalkStat
SET COMPATIBLE &lcCompStat
SET ESCAPE &lcEscStat
SET CURSOR &lcCurStat
SET TALK &lcTalkStat

* Return whether the array was "Changed" or "Unchanged" (lcResult
* was checked in ExitAedit() )

RETURN lcResult

*!*********************************************************************
*!
*!       Function: REDRAW()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!               : MOVERIGHT()        (function  in AEDIT.PRG)
*!               : MOVELEFT()         (function  in AEDIT.PRG)
*!               : MOVEDOWN()         (function  in AEDIT.PRG)
*!               : MOVEUP()           (function  in AEDIT.PRG)
*!               : SCREENHOME()       (function  in AEDIT.PRG)
*!               : SCREENEND()        (function  in AEDIT.PRG)
*!               : SCREENRIGHT()      (function  in AEDIT.PRG)
*!               : SCREENLEFT()       (function  in AEDIT.PRG)
*!               : SCREENUP()         (function  in AEDIT.PRG)
*!               : SCREENDOWN()       (function  in AEDIT.PRG)
*!
*!*********************************************************************

FUNCTION redraw

* PURPOSE
* -------
* Display a screenful of data
*
* PARAMETERS
* ----------
* lcTitles : If this parameter contains "V" the row titles are 
*            redrawn along with the rest of the array.
*            If this parameter contains "H" the column titles are 
*            redrawn along with the rest of the array.

PARAMETERS lcTitles


IF 'H' $ UPPER(lcTitles)
  FOR j=1 TO lnHCnt
    @ 1,lnLeftT+3+(j-1)*lnCellLen ;
      SAY PADC(laTopT[j+lnLeftCol-1],lnCellLen) ;
      COLOR (SCHEME(9,2))
  ENDFOR
ENDIF

IF 'V' $ UPPER(lcTitles)
  FOR i=1 TO lnVCnt
    @ i+2,2 SAY PADR(laLeftT[i+lnTopRow-1],lnLeftT) ;
      COLOR (SCHEME(9,2))
  ENDFOR
ENDIF

FOR i=1 TO lnVCnt
  FOR j=1 TO lnHCnt
    @ i+2,lnLeftT+3+(j-1)*lnCellLen SAY laArr[lnTopRow+i-1,lnLeftCol+j-1] ;
      PICTURE lcPicture COLOR (SCHEME(9,2))
  ENDFOR
ENDFOR

RETURN .T.

*!*********************************************************************
*!
*!       Function: DRAWCURSOR()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!               : MOVERIGHT()        (function  in AEDIT.PRG)
*!               : MOVELEFT()         (function  in AEDIT.PRG)
*!               : MOVEDOWN()         (function  in AEDIT.PRG)
*!               : MOVEUP()           (function  in AEDIT.PRG)
*!               : SCREENHOME()       (function  in AEDIT.PRG)
*!               : SCREENEND()        (function  in AEDIT.PRG)
*!               : SCREENRIGHT()      (function  in AEDIT.PRG)
*!               : SCREENLEFT()       (function  in AEDIT.PRG)
*!               : SCREENUP()         (function  in AEDIT.PRG)
*!               : SCREENDOWN()       (function  in AEDIT.PRG)
*!               : EDITCELL()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION drawCursor

* PURPOSE
* -------
* Display the cusor (a whole cell)
*

* Column title
@ 1,lnLeftT+3+(lnCol-1)*lnCellLen FILL TO ;
  1,lnLeftT+lnCol*lnCellLen+2 COLOR (SCHEME(9,6))

* Row title
@ lnRow+2,2 FILL TO lnRow+2,1+lnLeftT COLOR (SCHEME(9,6))

* Cursor itself
@ lnRow+2,lnLeftT+3+(lnCol-1)*lnCellLen FILL TO ;
  lnRow+2,lnLeftT+lnCol*lnCellLen+2 COLOR (SCHEME(9,6))

RETURN .T.

*!*********************************************************************
*!
*!       Function: CLEARCURSOR()
*!
*!      Called by: MOUSEPRESS()       (function  in AEDIT.PRG)
*!               : MOVERIGHT()        (function  in AEDIT.PRG)
*!               : MOVELEFT()         (function  in AEDIT.PRG)
*!               : MOVEDOWN()         (function  in AEDIT.PRG)
*!               : MOVEUP()           (function  in AEDIT.PRG)
*!               : SCREENHOME()       (function  in AEDIT.PRG)
*!               : SCREENEND()        (function  in AEDIT.PRG)
*!               : SCREENRIGHT()      (function  in AEDIT.PRG)
*!               : SCREENLEFT()       (function  in AEDIT.PRG)
*!               : SCREENUP()         (function  in AEDIT.PRG)
*!               : SCREENDOWN()       (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION clearCursor

* PURPOSE
* -------
* Clear the cursor
*

* Column title
@ 1,lnLeftT+3+(lnCol-1)*lnCellLen FILL TO ;
  1,lnLeftT+lnCol*lnCellLen+2 COLOR (SCHEME(9,2))

* Row title
@ lnRow+2,2 FILL TO lnRow+2,1+lnLeftT COLOR (SCHEME(9,2))

* Cursor itself
@ lnRow+2,lnLeftT+3+(lnCol-1)*lnCellLen FILL TO ;
  lnRow+2,lnLeftT+lnCol*lnCellLen+2 COLOR (SCHEME(9,2))

RETURN .T.

*!*********************************************************************
*!
*!       Function: DRAWELEV()
*!
*!      Called by: MOUSEPRESS()       (function  in AEDIT.PRG)
*!               : MOVERIGHT()        (function  in AEDIT.PRG)
*!               : MOVELEFT()         (function  in AEDIT.PRG)
*!               : MOVEDOWN()         (function  in AEDIT.PRG)
*!               : MOVEUP()           (function  in AEDIT.PRG)
*!               : SCREENHOME()       (function  in AEDIT.PRG)
*!               : SCREENEND()        (function  in AEDIT.PRG)
*!               : SCREENRIGHT()      (function  in AEDIT.PRG)
*!               : SCREENLEFT()       (function  in AEDIT.PRG)
*!               : SCREENUP()         (function  in AEDIT.PRG)
*!               : SCREENDOWN()       (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION drawElev

* PURPOSE
* -------
* Display the elevators
*
* PARAMETERS
* ----------
* lcHV     : If this parameter contains "V" the vertical elevator
*            is redrawn.
*            If this parameter contains "H" the horizontal elevator
*            is redrawn.

PARAMETERS lcHV

IF 'H' $ UPPER(lcHV)
  IF lnHCnt<ALEN(laArr,2)
    lnHElev=FLOOR((WCOLS()-5)*(lnLeftCol+lnCol-2)/(ALEN(laArr,2)-1))
    MOVE WINDOW wHElev TO WROWS()-1,2+lnHElev
  ENDIF
ENDIF

IF 'V' $ UPPER(lcHV)
  IF lnVCnt<ALEN(laArr,1)
    lnVElev=FLOOR((WROWS()-5)*(lnTopRow+lnRow-2)/(ALEN(laArr,1)-1))
    MOVE WINDOW wVElev TO 2+lnVElev,WCOLS()-1
  ENDIF
ENDIF

RETURN .T.

*!*********************************************************************
*!
*!       Function: EDITCELL()
*!
*!      Called by: AEDIT.PRG                         
*!
*!          Calls: DRAWCURSOR()       (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION editCell

* PURPOSE
* -------
* GET the current cell
*

SET CURSOR ON
@ lnRow+2,lnLeftT+3+(lnCol-1)*lnCellLen ;
  GET laArr[lnRow+lnTopRow-1,lnCol+lnLeftCol-1] PICTURE lcPicture
READ
SET CURSOR OFF
=drawCursor()

RETURN .T.

*!*********************************************************************
*!
*!       Function: MOVELEFT()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: REDRAW()           (function  in AEDIT.PRG)
*!               : CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION moveLeft

* PURPOSE
* -------
* Move the cursor one cell left
*

IF lnCol=1
  IF (lnLeftCol>1)
    lnLeftCol=lnLeftCol-1
    =redraw("H")
  ENDIF
ELSE
  =ClearCursor()
  lnCol=lnCol-1
ENDIF

=drawCursor()
=drawElev("H")

RETURN .T.

*!*********************************************************************
*!
*!       Function: MOVERIGHT()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: REDRAW()           (function  in AEDIT.PRG)
*!               : CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION moveRight

* PURPOSE
* -------
* Move the cursor one cell right
*

IF lnCol=lnHCnt
  IF (lnLeftCol+lnHCnt-1<ALEN(laArr,2))
    lnLeftCol=lnLeftCol+1
    =redraw("H")
  ENDIF
ELSE
  =ClearCursor()
  lnCol=lnCol+1
ENDIF

=drawCursor()
=drawElev("H")

RETURN .T.

*!*********************************************************************
*!
*!       Function: MOVEUP()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: REDRAW()           (function  in AEDIT.PRG)
*!               : CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION moveUp

* PURPOSE
* -------
* Move the cursor one cell up
*

IF lnRow=1
  IF lnTopRow>1
    lnTopRow=lnTopRow-1
    =redraw("V")
  ENDIF
ELSE
  =ClearCursor()
  lnRow=lnRow-1
ENDIF

=drawCursor()
=drawElev("V")

RETURN .T.

*!*********************************************************************
*!
*!       Function: MOVEDOWN()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: REDRAW()           (function  in AEDIT.PRG)
*!               : CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION moveDown

* PURPOSE
* -------
* Move the cursor one cell down
*

IF lnRow=lnVCnt
  IF (lnTopRow+lnVCnt-1<ALEN(laArr,1))
    lnTopRow=lnTopRow+1
    =redraw("V")
  ENDIF
ELSE
  =ClearCursor()
  lnRow=lnRow+1
ENDIF

=drawCursor()
=drawElev("V")

RETURN .T.

*!*********************************************************************
*!
*!       Function: SCREENHOME()
*!
*!      Called by: AEDIT.PRG                         
*!
*!          Calls: CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : REDRAW()           (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION screenHome

* PURPOSE
* -------
* Move the cursor to the first cell of the first line
*

PRIVATE lcTemp

lcTemp=""
IF lnLeftCol>1
  lcTemp="H"
ENDIF
IF lnTopRow>1
  lcTemp=lcTemp+"V"
ENDIF

=ClearCursor()
lnTopRow=1
lnLeftCol=1
lnRow=1
lnCol=1

IF ! EMPTY(lcTemp)
  =redraw(lcTemp)
ENDIF

=drawCursor()
=drawElev(lcTemp)

RETURN .T.

*!*********************************************************************
*!
*!       Function: SCREENEND()
*!
*!      Called by: AEDIT.PRG                         
*!
*!          Calls: CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : REDRAW()           (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION screenEnd

* PURPOSE
* -------
* Move the cursor to the last cell of the last line
*

PRIVATE lcTemp

lctemp=""
IF lnLeftCol<ALEN(laArr,2)-lnHCnt+1
  lcTemp="H"
ENDIF
IF lnTopRow<ALEN(laArr,1)-lnVCnt+1
  lcTemp=lcTemp+"V"
ENDIF

=ClearCursor()
lnTopRow=ALEN(laArr,1)-lnVCnt+1
lnLeftCol=ALEN(laArr,2)-lnHCnt+1
lnRow=lnVCnt
lnCol=lnHCnt

IF ! EMPTY(lcTemp)
  =redraw(lcTemp)
ENDIF

=drawCursor()
=drawElev(lcTemp)

RETURN .T.

*!*********************************************************************
*!
*!       Function: SCREENLEFT()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : REDRAW()           (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION screenLeft

* PURPOSE
* -------
* Move the cursor one screen left
*

IF lnCol>1
  =clearCursor()
  lnCol=1
ELSE
  lnLeftCol=MAX(1,lnLeftCol-lnHCnt+1)
  =redraw("H")
ENDIF

=drawCursor()
=drawElev("H")

RETURN .T.

*!*********************************************************************
*!
*!       Function: SCREENRIGHT()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : REDRAW()           (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION screenRight

* PURPOSE
* -------
* Move the cursor one screen right
*

IF lnCol<lnHCnt
  =clearCursor()
  lnCol=lnHCnt
ELSE
  lnLeftCol=MIN(ALEN(laArr,2)-lnHCnt+1,lnLeftCol+lnHCnt-1)
  =redraw("H")
ENDIF

=drawCursor()
=drawElev("H")

RETURN .T.

*!*********************************************************************
*!
*!       Function: SCREENUP()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : REDRAW()           (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION screenUp

* PURPOSE
* -------
* Move the cursor one screen up
*

IF lnRow>1
  =clearCursor()
  lnRow=1
ELSE
  lnTopRow=MAX(1,lnTopRow-lnVCnt+1)
  =redraw("V")
ENDIF

=drawCursor()
=drawElev("V")

RETURN .T.

*!*********************************************************************
*!
*!       Function: SCREENDOWN()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!          Calls: CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : REDRAW()           (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION screenDown

* PURPOSE
* -------
* Move the cursor one screen down
*

IF lnRow<lnVCnt
  =clearCursor()
  lnRow=lnVCnt
ELSE
  lnTopRow=MIN(ALEN(laArr,1)-lnVCnt+1,lnTopRow+lnVCnt-1)
  =redraw("V")
ENDIF

=drawCursor()
=drawElev("V")

RETURN .T.

*!*********************************************************************
*!
*!       Function: MOUSEPRESS()
*!
*!      Called by: AEDIT.PRG                         
*!
*!          Calls: CLEARCURSOR()      (function  in AEDIT.PRG)
*!               : REDRAW()           (function  in AEDIT.PRG)
*!               : DRAWCURSOR()       (function  in AEDIT.PRG)
*!               : DRAWELEV()         (function  in AEDIT.PRG)
*!               : MOVELEFT()         (function  in AEDIT.PRG)
*!               : MOVERIGHT()        (function  in AEDIT.PRG)
*!               : SCREENLEFT()       (function  in AEDIT.PRG)
*!               : SCREENRIGHT()      (function  in AEDIT.PRG)
*!               : MOVEUP()           (function  in AEDIT.PRG)
*!               : MOVEDOWN()         (function  in AEDIT.PRG)
*!               : SCREENUP()         (function  in AEDIT.PRG)
*!               : SCREENDOWN()       (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION mousePress

* PURPOSE
* -------
* Handle mouse clicks and drag events
*

PRIVATE lnMRow     && Row position of mouse-click
PRIVATE lnMCol     && Col position of mouse-click
PRIVATE lnNewRow   && New array index of the cursor row 
                   && after a mouse drag.
PRIVATE lnNewCol   && New array index of the cursor column
                   && after a mouse drag.

* Store mouse position
lnMRow=MROW()
lnMCol=MCOL()

* Click on row 0, col 0: Close window
IF (lnMRow=0) .AND. (lnMCol=0)
  =exitAedit()
  RETURN .T.
ENDIF

* Click on another position of row 0: Drag window
IF (lnMRow=0) .AND. BETWEEN(lnMCol,1,WCOLS()-2)
  DO WHILE MDOWN()
    MOVE WINDOW editArray TO MROW(''),MCOL('')-lnMCol
  ENDDO
  RETURN .T.
ENDIF

* Click somewhere on the horizontal elevator
IF (lnMRow=WROWS()-1) .AND. (lnHCnt<ALEN(laArr,2))
  DO CASE
		* Click on the horizontal elevator: drag elevator
    CASE (lnMCol=lnHElev+2)
      * Lift elevator
      DO WHILE MDOWN()
        MOVE WINDOW wHElev TO WROWS()-1,MIN(WCOLS()-3,MAX(MCOL(),2))
      ENDDO
      * Compute the new array column index for the cursor
      lnNewCol=FLOOR(1+(ALEN(laArr,2)-1)*MIN(WCOLS()-5,MAX(MCOL()-2,0))/(WCOLS()-5))
      =ClearCursor()
      * Adjust display and cursor position 
      IF BETWEEN(lnNewCol,lnLeftCol,lnLeftCol+lnHCnt-1)
        lnCol=lnNewCol-lnLeftCol+1
      ELSE
        IF lnNewCol>ALEN(laArr,2)-lnHCnt+1
          lnLeftCol=ALEN(laArr,2)-lnHCnt+1
          lnCol=lnNewCol-lnLeftCol+1
        ELSE
          lnLeftCol=lnNewCol
          lnCol=1
        ENDIF
        =redraw("H")
      ENDIF
      =drawCursor()
      =drawElev("H")
    * Click on left elevator arrow: Move one cell left
    CASE lnMCol=1
      =moveLeft()
    * Click on right elevator arrow: Move one cell right
    CASE lnMCol=WCOLS()-2
      =moveRight()
    * Click between the left elevator arrow and
    * the elevator itself: Move one screen left
    CASE BETWEEN(lnMCol,2,lnHElev)
      =screenLeft()
    * Click between the right elevator arrow and
    * the elevator itself: Move one screen right
    CASE BETWEEN(lnMCol,2+lnHElev,WCOLS()-3)
      =screenRight()
  ENDCASE
  RETURN .T.
ENDIF


* Click somewhere on the vertical elevator
IF (lnMCol=WCOLS()-1) .AND. (lnVCnt<ALEN(laArr,1))
  DO CASE
		* Click on the vertical elevator: drag elevator
    CASE (lnMRow=lnVElev+2)
      * Lift elevator
      DO WHILE MDOWN()
        MOVE WINDOW wVElev TO MIN(WROWS()-3,MAX(MROW(),2)),WCOLS()-1
      ENDDO
      * Compute the new array row index for the cursor
      lnNewRow=FLOOR(1+(ALEN(laArr,1)-1)*MIN(WROWS()-5,MAX(MROW()-2,0))/(WROWS()-5))
      =ClearCursor()
      * Adjust display and cursor position 
      IF BETWEEN(lnNewRow,lnTopRow,lnTopRow+lnVCnt-1)
        lnRow=lnNewRow-lnTopRow+1
      ELSE
        IF lnNewRow>ALEN(laArr,1)-lnVCnt+1
          lnTopRow=ALEN(laArr,1)-lnVCnt+1
          lnRow=lnNewRow-lnTopRow+1
        ELSE
          lnTopRow=lnNewRow
          lnRow=1
        ENDIF
        =redraw("V")
      ENDIF
      =drawCursor()
      =drawElev("V")
    * Click on top elevator arrow: Move one cell up
    CASE lnMRow=1
      =moveUp()
    * Click on bottom elevator arrow: Move one cell down
    CASE lnMRow=WROWS()-2
      =moveDown()
    * Click between the top elevator arrow and
    * the elevator itself: Move one screen up
    CASE BETWEEN(lnMRow,2,lnVElev+1)
      =screenUp()
    * Click between the bottom elevator arrow and
    * the elevator itself: Move one screen down
    CASE BETWEEN(lnMRow,3+lnVElev,WROWS()-3)
      =screenDown()
  ENDCASE
  RETURN .T.
ENDIF

* Click on a cell: Edit it
IF BETWEEN(lnMRow,3,WROWS()-2)
  IF BETWEEN(lnMCol,lnLeftT+3,WCOLS()-2)
    =clearCursor()
    lnRow=lnMrow-2
    lnCol=CEILING((lnMCol-lnLeftT-2)/lnCellLen)
    =drawElev("HV")
    =drawCursor()
  ENDIF
ENDIF

RETURN .T.


*!*********************************************************************
*!
*!       Function: EXITAEDIT()
*!
*!      Called by: AEDIT.PRG                         
*!               : MOUSEPRESS()       (function  in AEDIT.PRG)
*!
*!*********************************************************************
FUNCTION ExitAedit

PRIVATE lcAnswer

* Compare the array with the backup copy
FOR i=1 TO ALEN(laArr,0)
  IF laArr[i]<>laBackup[i]
    lcResult="Changed"
    EXIT
  ENDIF
ENDFOR

* If the array was modified, ask the user to confirm his changes
IF (lcResult="Changed")
	  lcAnswer="Cancel"
    DEFINE WINDOW wmAskUser IN SCREEN ;
    FROM INT((SROW()-7)/2),INT((SCOL()-55)/2) ;
    TO INT((SROW()-7)/2)+6,INT((SCOL()-55)/2)+54 ;
    FLOAT ;
    NOCLOSE ;
    PANEL ;
    COLOR SCHEME 5
  ACTIVATE WINDOW wmAskUser
	  @ 1,2 SAY "Do you want to save your changes before exiting?"
  	@ 3,2 GET lcAnswer ;
    	PICTURE "@*HT \!\<Yes;\<No;\?\<Cancel" ;
    	SIZE 1,7,5
  	READ CYCLE MODAL
  RELEASE WINDOW wmAskUser
  DO CASE
  	CASE lcAnswer="Cancel"
  		* Nothing...
  	CASE lcAnswer="Yes"
  	  llExit=.T.
  	CASE lcAnswer="No"
  	  =ACOPY(laBackup,laArr)
    	lcResult="Unchanged"
    	llExit=.T.
  ENDCASE
ELSE
	llExit=.T.
ENDIF

RETURN .T.

*: EOF: AEDIT.PRG
